import { HTMLProps, ReactNode, SyntheticEvent } from 'react';
export interface DateObj {
    date: Date;
    nextMonth: boolean;
    prevMonth: boolean;
    selectable: boolean;
    selected: boolean;
    today: boolean;
}
export interface Calendar {
    firstDayOfMonth: Date;
    lastDayOfMonth: Date;
    month: 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11;
    weeks: Array<Array<DateObj | ''>>;
    year: number;
}
export interface GetBackForwardPropsOptions extends HTMLProps<HTMLButtonElement> {
    calendars: Calendar[];
    offset?: number;
}
export interface GetDatePropsOptions extends HTMLProps<HTMLButtonElement> {
    dateObj: DateObj;
}
export interface RenderProps {
    calendars: Calendar[];
    getBackProps: (data: GetBackForwardPropsOptions) => Record<string, any>;
    getForwardProps: (data: GetBackForwardPropsOptions) => Record<string, any>;
    getDateProps: (data: GetDatePropsOptions) => Record<string, any>;
}
export type RenderFn = (renderProps: RenderProps) => ReactNode;
export interface Props {
    date?: Date;
    maxDate?: Date;
    minDate?: Date;
    monthsToDisplay?: number;
    firstDayOfWeek?: 0 | 1 | 2 | 3 | 4 | 5 | 6;
    showOutsideDays?: boolean;
    selected?: Date | Date[];
    children?: RenderFn;
    render?: RenderFn;
    offset?: number;
    onOffsetChanged?(offset: number): void;
    onDateSelected(selectedDate: DateObj, event: SyntheticEvent): void;
}
/**
 * This is intended to be used to compose event handlers
 * They are executed in order until one of them calls
 * `event.preventDefault()`. Not sure this is the best
 * way to do this, but it seems legit...
 * @param {Function} fns the event hanlder functions
 * @return {Function} the event handler to add to an element
 */
export declare function composeEventHandlers(...fns: Function[]): (event: any, ...args: any[]) => boolean;
/**
 * Takes an argument and if it's an array, returns the first item in the array
 * otherwise returns the argument.
 * @param {*} arg the maybe-array
 * @return {*} the arg or it's first item
 */
export declare function unwrapChildrenForPreact(arg: any): any;
/**
 * Takes a calendars array and figures out the number of months to subtract
 * based on the current offset and the minDate allowed.
 * @param {Object} param The param object
 * @param {Array} param.calendars The calendars array created by the getCalendars function
 * @param {Number} param.offset The num of months to be subtracted
 * @param {Date} param.minDate The earliest date we are allow to subtract back to
 * @returns {Number} The number of months to subtract
 */
export declare function subtractMonth({ calendars, offset, minDate, }: {
    calendars: Calendar[];
    offset: number;
    minDate: Date | undefined;
}): number;
/**
 * Takes a calendars array and figures out the number of months to add
 * based on the current offset and the maxDate allowed.
 * @param {Object} param The param object
 * @param {Array} param.calendars The calendars array created by the getCalendars function
 * @param {Number} param.offset The num of months to be added
 * @param {Date} param.maxDate The furthest date we are allow to add forward to
 * @returns {Number} The number of months to add
 */
export declare function addMonth({ calendars, offset, maxDate, }: {
    calendars: Calendar[];
    offset: number;
    maxDate: Date | undefined;
}): number;
/**
 * Takes a calendars array and figures out if the back button should be
 * disabled based on the minDate allowed.
 * @param {Object} param The param object
 * @param {Array} param.calendars The calendars array created by the getCalendars function
 * @param {Date} param.minDate The earliest date available
 * @returns {Boolean} Whether the back button should be disabled.
 */
export declare function isBackDisabled({ calendars, minDate, }: {
    calendars: Calendar[];
    minDate: Date | undefined;
}): boolean;
/**
 * Takes a calendars array and figures out if the forward button should be
 * disabled based on the maxDate allowed.
 * @param {Object} param The param object
 * @param {Array} param.calendars The calendars array created by the getCalendars function
 * @param {Date} param.maxDate The furthest date available
 * @returns {Boolean} Whether the forward button should be disabled.
 */
export declare function isForwardDisabled({ calendars, maxDate, }: {
    calendars: Calendar[];
    maxDate: Date | undefined;
}): boolean;
/**
 * Figures out the months data needed based off the number of monthsToDisplay
 * and other options provided.
 * @param {Object} param The param object
 * @param {Date} param.date The date to start the calendar at
 * @param {Array.<Date>} param.selected An array of dates currently selected
 * @param {Number} param.monthsToDisplay The number of months to return in the calendar view
 * @param {Number} param.offset The number of months to offset based off the param.date given
 * @param {Date} param.minDate The earliest date available
 * @param {Date} param.maxDate The furthest date available
 * @param {Number} param.firstDayOfWeek First day of week, 0-6 (Sunday to Saturday)
 * @param {Bool} param.showOutsideDays Flag to fill front and back weeks with dates from adjacent months
 * @returns {Array.<Object>} An array of objects with month data
 */
export declare function getCalendars({ date, selected, monthsToDisplay, offset, minDate, maxDate, firstDayOfWeek, showOutsideDays, }: {
    date: Date;
    selected: Date | Date[] | undefined;
    monthsToDisplay: number;
    offset: number;
    minDate: Date | undefined;
    maxDate: Date | undefined;
    firstDayOfWeek: number;
    showOutsideDays: boolean;
}): Calendar[];
